/**
 * Copyright 2010-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.auditreport.impl;

import org.ow2.util.auditreport.api.IAuditID;
import org.ow2.util.auditreport.api.ICurrentInvocationID;

/**
 * Allows to get the current ID.
 * @author Florent Benoit
 */
public class CurrentInvocationID implements ICurrentInvocationID {

    /**
     * Inherited Local thread used to keep the ID.
     */
    private ThreadLocal<IAuditID> threadLocal = null;

    /**
     * Default constructor.
     */
    public CurrentInvocationID() {
        this.threadLocal = new ThreadLocal<IAuditID>();
    }

    /**
     * Unique instance of this current object.
     */
    private static ICurrentInvocationID unique = null;

    /**
     * Return the unique instance of this object.
     * @return SecurityCurrent return the current
     */
    public static synchronized ICurrentInvocationID getInstance() {
        if (unique == null) {
            // Build a default implementation
            unique = new CurrentInvocationID();
        }
        return unique;
    }

    /**
     * Associates the given audit ID to the current thread.
     * @param auditID the ID to associate to the current thread.
     * @return the previous ID
     */
    public IAuditID setAuditID(final IAuditID auditID) {
        IAuditID previous = threadLocal.get();
        threadLocal.set(auditID);
        return previous;
    }

    /**
     * Gets the current audit ID (if any).
     * @return the ID to associate to the current thread
     */
    public IAuditID getAuditID() {
        return threadLocal.get();
    }

    /**
     * Initialize a new Audit ID with an optional value.
     * @param id the initial value
     */
    public void init(final IAuditID id) {
        IAuditID newId = null;
        if (id != null) {
            newId = id;
        } else {
            newId = new AuditIDImpl();
            newId.generate();
        }
        threadLocal.set(newId);
    }

    /**
     * Generate a new ID and return the previously set.
     * @return the previous ID.
     */
    public IAuditID newInvocation() {
        IAuditID previous = threadLocal.get();
        AuditIDImpl id = new AuditIDImpl();
        id.generate();
        if (previous != null) {
            previous.increment();
            id.setParentID(previous.getLocalID());
        }
        threadLocal.set(id);
        return previous;
    }




}
