/**
 * Copyright 2010-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.auditreport.impl;

import org.ow2.util.auditreport.impl.util.ObjectEncoder;

/**
 * Report for EJB invocations events.
 * @author Mathieu ANCELIN
 */
public class InvocationAuditReport extends GenericAuditReport {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -3332882932887333168L;

    /**
     * The time the request start.
     */
    private long requestStart = ZERO; // in nanoseconds

    /**
     * The time the request stop.
     */
    private long requestStop = ZERO; // in nanoseconds

    /**
     * The request duration time.
     */
    private double requestDuration = ZERO; // in nanoseconds

    /**
     * The request business method.
     */
    private String businessMethod = EMPTY_STRING;

    /**
     * The request target.
     */
    private String target = EMPTY_STRING;

    /**
     * The request parameters.
     */
    private String[] methodParameters = null;

    /**
     * The request return.
     */
    private Object methodReturn = null;

    /**
     * Thte size of the parameters.
     */
    private long paramSize = ZERO;

    /**
     * Thte size of the parameters.
     */
    private long returnSize = ZERO;

    static {
        // Register fields "StackTraceElement[] methodStackTrace".
        ObjectEncoder.registerField(InvocationAuditReport.class, "methodParameters", EncodingType.OBJECTTABLE);
        ObjectEncoder.registerField(InvocationAuditReport.class, "methodReturn", EncodingType.OBJECT);
    }

    /**
     * Default constructor.
     */
    public InvocationAuditReport() {
        super();
    }

    /**
     * Constructor of the report.
     * @param requestStartParam time the request start.
     * @param requestTimeStampParam request timestamp.
     * @param businessMethodParam requested method.
     * @param targetParam target of the request.
     * @param methodParametersParam params of the request.
     * @param currentThreadParam the current thread.
     * @param freeMemoryBeforeParam free memory before the call.
     * @param totalMemoryBeforeParam total memory before the call.
     * @param roles the security roles.
     * @param user the user identity
     * @param paramSize size of paramaters.
     */
    public InvocationAuditReport(final long requestStartParam, final long requestTimeStampParam,
            final String businessMethodParam, final String targetParam, final String[] methodParametersParam,
            final Thread currentThreadParam, final int freeMemoryBeforeParam, final int totalMemoryBeforeParam,
            final String[] roles, final String user, final int paramSize) {
        super();
        setRequestTimeStamp(requestTimeStampParam);
        setFreeMemoryBefore(freeMemoryBeforeParam);
        setTotalMemoryBefore(totalMemoryBeforeParam);
        if (roles != null) {
            setRoles(java.util.Arrays.asList(roles).toString());
        }
        setUser(user);
        this.paramSize = paramSize;
        this.target = targetParam;
        this.businessMethod = businessMethodParam;
        this.methodParameters = methodParametersParam;
        this.requestStart = requestStartParam;
    }

    /**
     * @return businessMethod the businessMethod to get
     */
    public String getBusinessMethod() {
        return this.businessMethod;
    }

    /**
     * @return methodParameters the methodParameters to get
     */
    public String[] getMethodParameters() {
        return this.methodParameters;
    }

    /**
     * @return methodReturn the methodReturn to get
     */
    public Object getMethodReturn() {
        return this.methodReturn;
    }

    /**
     * @return requestDuration the requestDuration to get
     */
    public double getRequestDuration() {
        return this.requestDuration;
    }

    /**
     * @return requestStart the requestStart to get
     */
    public long getRequestStart() {
        return this.requestStart;
    }

    /**
     * @return requestStop the requestStop to get
     */
    public long getRequestStop() {
        return this.requestStop;
    }

    /**
     * @param requestStartParam the requestStart to set
     */
    public void setRequestStart(final long requestStartParam) {
        this.requestStart = requestStartParam;
    }

    /**
     * @param requestStopParam the requestStop to set
     */
    public void setRequestStop(final long requestStopParam) {
        this.requestStop = requestStopParam;
        this.requestDuration = (requestStopParam - this.requestStart);
    }

    /**
     * @param requestDurationParam the requestDuration to set
     */
    public void setRequestDuration(final double requestDurationParam) {
        this.requestDuration = requestDurationParam;
    }

    /**
     * @param businessMethodParam the businessMethod to set
     */
    public void setBusinessMethod(final String businessMethodParam) {
        this.businessMethod = businessMethodParam;
    }

    /**
     * @return the target
     */
    public String getTarget() {
        return this.target;
    }

    /**
     * @param targetParam the target to set
     */
    public void setTarget(final String targetParam) {
        this.target = targetParam;
    }

    /**
     * @param methodReturnParam the methodReturn to set
     */
    public void setMethodReturn(final Object methodReturnParam) {
        this.methodReturn = methodReturnParam;
    }

    /**
     * @param methodParametersParam the method parameters
     */
    public void setMethodParameters(final String[] methodParametersParam) {
        this.methodParameters = methodParametersParam;
    }

    /**
     * @return the paramSize
     */
    public long getParamSize() {
        return this.paramSize;
    }

    /**
     * @param paramSize the paramSize to set
     */
    public void setParamSize(final long paramSize) {
        this.paramSize = paramSize;
    }

    /**
     * @return the returnSize
     */
    public long getReturnSize() {
        return this.returnSize;
    }

    /**
     * @param returnSize the returnSize to set
     */
    public void setReturnSize(final long returnSize) {
        this.returnSize = returnSize;
    }
}
