/**
 * Copyright 2010-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.auditreport.impl;

import org.ow2.util.auditreport.impl.util.ObjectEncoder;

/**
 * Report for JNDI invocations events.
 * @author Mathieu ANCELIN
 */
public class JNDIAuditReport extends GenericAuditReport {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 1935408900498244921L;

    /**
     * The time the request start.
     */
    private long requestStart = ZERO; // in nanoseconds

    /**
     * The time the request stop.
     */
    private long requestStop = ZERO; // in nanoseconds

    /**
     * The request duration time.
     */
    private double requestDuration = ZERO; // in milliseconds

    /**
     * The request business method.
     */
    private String businessMethod = EMPTY_STRING;

    /**
     * The request target.
     */
    private String target = EMPTY_STRING;

    /**
     * The request parameters.
     */
    private String[] contextParameters = null;

    static {
        // Register fields "StackTraceElement[] methodStackTrace".
        ObjectEncoder.registerField(JNDIAuditReport.class, "contextParameters", EncodingType.OBJECTTABLE);
    }

    /**
     * Default constructor.
     */
    public JNDIAuditReport() {
        super();
    }

    /**
     * Constructor of the report.
     * @param requestTimeStampParam request timestamp.
     * @param businessMethodParam requested method.
     * @param targetParam target of the request.
     * @param contextParameters the parameters of the context.
     * @param currentThreadParam the current thread.
     */
    public JNDIAuditReport(final long requestTimeStampParam, final String businessMethodParam, final String targetParam,
            final String[] contextParameters, final Thread currentThreadParam, final long start, final long stop) {
        super();
        int freeMemory = (int) (Runtime.getRuntime().freeMemory());
        int totalMemory = (int) (Runtime.getRuntime().totalMemory());
        this.setRequestTimeStamp(requestTimeStampParam);
        this.setFreeMemoryBefore(freeMemory);
        this.setTotalMemoryBefore(totalMemory);
        this.setFreeMemoryAfter(freeMemory);
        this.setTotalMemoryAfter(totalMemory);
        this.setRoles("jndi");
        this.target = targetParam;
        this.businessMethod = businessMethodParam;
        this.contextParameters = contextParameters;
        this.requestStart = start;
        this.requestStop = stop;
        this.requestDuration = (requestStop - requestStart) / NS;
    }

    /**
     * @return the businessMethod
     */
    public String getBusinessMethod() {
        return businessMethod;
    }

    /**
     * @param businessMethod the businessMethod to set
     */
    public void setBusinessMethod(final String businessMethod) {
        this.businessMethod = businessMethod;
    }

    /**
     * @return the target
     */
    public String getTarget() {
        return target;
    }

    /**
     * @param target the target to set
     */
    public void setTarget(final String target) {
        this.target = target;
    }

    /**
     * @return the contextParameters
     */
    public String[] getContextParameters() {
        return contextParameters;
    }

    /**
     * @param contextParameters the contextParameters to set
     */
    public void setContextParameters(final String[] contextParameters) {
        this.contextParameters = contextParameters;
    }

    /**
     * @return the duration of the request.
     */
    public double getRequestDuration() {
        return requestDuration;
    }

    /**
     * @param requestDuration the duration of the request.
     */
    public void setRequestDuration(final double requestDuration) {
        this.requestDuration = requestDuration;
    }

    /**
     * @return the start time of the request.
     */
    public long getRequestStart() {
        return requestStart;
    }

    /**
     * @param requestStart the start time of the request.
     */
    public void setRequestStart(final long requestStart) {
        this.requestStart = requestStart;
    }

    /**
     * @return the stop time of the request.
     */
    public long getRequestStop() {
        return requestStop;
    }

    /**
     * @param requestStop the stop time of the request.
     */
    public void setRequestStop(final long requestStop) {
        this.requestStop = requestStop;
    }

}
