/**
 * Copyright 2010-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.auditreport.impl;

import javax.servlet.http.HttpServletRequest;

import org.ow2.util.auditreport.impl.util.ObjectEncoder;

/**
 * Report for WEB invocations events.
 * @author Mathieu ANCELIN
 */
public class WebInvocationAuditReport extends GenericAuditReport {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 4264755180723751875L;

    /**
     * The time the request start.
     */
    private long requestStart = ZERO; // in nanoseconds

    /**
     * The time the request stop.
     */
    private long requestStop = ZERO; // in nanoseconds

    /**
     * The request duration time.
     */
    private double requestDuration = ZERO; // in nanoseconds

    /**
     * The request business method.
     */
    private String businessMethod = EMPTY_STRING;

    /**
     * The request target.
     */
    private String target = EMPTY_STRING;

    /**
     * The request parameters.
     */
    private Object[] methodParameters = null;

    /**
     * The request return.
     */
    private Object methodReturn = null;

    /**
     * Thte size of the parameters.
     */
    private long paramSize = ZERO;

    /**
     * Thte size of the parameters.
     */
    private long returnSize = ZERO;

    /**
     * The authentification type.
     */
    private String authType = EMPTY_STRING;

    /**
     * The context path.
     */
    private String contextPath = EMPTY_STRING;

    /**
     * The path info.
     */
    private String pathInfo = EMPTY_STRING;

    /**
     * The translated path.
     */
    private String pathTranslated = EMPTY_STRING;

    /**
     * The query.
     */
    private String queryString = EMPTY_STRING;

    /**
     * The session ID.
     */
    private String requestedSessionId = EMPTY_STRING;

    /**
     * The URI of the request.
     */
    private String requestURI = EMPTY_STRING;

    /**
     * The URL of the request.
     */
    private String requestURL = EMPTY_STRING;

    /**
     * The servlet path.
     */
    private String servletPath = EMPTY_STRING;

    /**
     * The encoding.
     */
    private String characterEncoding = EMPTY_STRING;

    /**
     * The lenght of the content.
     */
    private int contentLength = ZERO;

    /**
     * The type of the content.
     */
    private String contentType = EMPTY_STRING;

    /**
     * The local address.
     */
    private String localAddress = EMPTY_STRING;

    /**
     * The local name.
     */
    private String localName = EMPTY_STRING;

    /**
     * The local port.
     */
    private int localPort = ZERO;

    /**
     * The protocol used.
     */
    private String protocol = EMPTY_STRING;

    /**
     * The remote address.
     */
    private String remoteAddress = EMPTY_STRING;

    /**
     * The remote host.
     */
    private String remoteHost = EMPTY_STRING;

    /**
     * The remote port.
     */
    private String remotePort = EMPTY_STRING;

    /**
     * The sheme of the request.
     */
    private String scheme = EMPTY_STRING;

    /**
     * The name of the server.
     */
    private String serverName = EMPTY_STRING;

    /**
     * The port of the server.
     */
    private int serverport = ZERO;

    /**
     * If the request is secured.
     */
    private boolean secure = false;

    /**
     * Status.
     */
    private int status = -1;

    static {
        // Register fields "StackTraceElement[] methodStackTrace".
        ObjectEncoder.registerField(WebInvocationAuditReport.class, "methodParameters", EncodingType.OBJECTTABLE);
        ObjectEncoder.registerField(WebInvocationAuditReport.class, "methodReturn", EncodingType.OBJECT);
    }

    /**
     * Default constructor.
     */
    public WebInvocationAuditReport() {
        super();
    }

    /**
     * Constructor of the report.
     * @param requestStartParam time the request start.
     * @param requestTimeStampParam request timestamp.
     * @param businessMethodParam requested method.
     * @param targetParam target of the request.
     * @param methodParametersParam params of the request.
     * @param currentThreadParam the current thread.
     * @param freeMemoryBeforeParam free memory before the call.
     * @param totalMemoryBeforeParam total memory before the call.
     * @param role the security role.
     * @param paramSize size of paramaters.
     */
    public WebInvocationAuditReport(final long requestStartParam, final long requestTimeStampParam,
            final String businessMethodParam, final String targetParam, final Object[] methodParametersParam,
            final Thread currentThreadParam, final int freeMemoryBeforeParam, final int totalMemoryBeforeParam,
            final String role, final int paramSize) {
        super();
        this.setRequestTimeStamp(requestTimeStampParam);
        this.setMethodStackTrace(currentThreadParam.getStackTrace());
        this.setFreeMemoryBefore(freeMemoryBeforeParam);
        this.setTotalMemoryBefore(totalMemoryBeforeParam);
        this.setRoles(role);
        this.setParamSize(paramSize);
        this.target = targetParam;
        this.businessMethod = businessMethodParam;
        this.methodParameters = methodParametersParam;
        this.requestStart = requestStartParam;
    }

    /**
     * Constructor from a web request.
     * @param req web request.
     */
    public WebInvocationAuditReport(final HttpServletRequest req) {
        super();
        this.setBusinessMethod(req.getMethod());
        this.setAuthType(req.getAuthType());
        this.setBeanName(req.getRequestURL().toString());
        // this.setCallLevel(level);
        this.setCharacterEncoding(req.getCharacterEncoding());
        this.setContentLength(req.getContentLength());
        this.setContentType(req.getContentType());
        this.setContextPath(req.getContextPath());
        // this.setFreeMemoryBefore(freeMemoryBefore);
        this.setLocalAddress(req.getLocalAddr());
        this.setLocalName(req.getLocalName());
        this.setLocalPort(req.getLocalPort());
        // this.setMethodParameters(test);
        // this.setMethodStackTrace(current.getStackTrace());
        this.setParamSize(req.getParameterMap().size());
        this.setPathInfo(req.getPathInfo());
        this.setPathTranslated(req.getPathTranslated());
        this.setProtocol(req.getProtocol());
        this.setQueryString(req.getQueryString());
        this.setRemoteAddress(req.getRemoteAddr());
        this.setRemoteHost(req.getRemoteHost());
        this.setRemotePort(String.valueOf(req.getRemotePort()));
        // this.setRequestStart(start);
        // this.setRequestTimeStamp(timestamp);
        this.setRequestURI(req.getRequestURI());
        this.setRequestURL(req.getRequestURL().toString());
        this.setRequestedSessionId(req.getRequestedSessionId());
        // this.setRole(role);
        this.setScheme(req.getScheme());
        this.setSecure(req.isSecure());
        this.setServerName(req.getServerName());
        this.setServerport(req.getServerPort());
        this.setServletPath(req.getServletPath());
        // this.setTarget(prefix + req.getRequestURI() + post);
        // this.setTotalMemoryBefore(totalMemoryBefore);
    }

    /**
     * @return businessMethod the businessMethod to get
     */
    public String getBusinessMethod() {
        return businessMethod;
    }

    /**
     * @return methodParameters the methodParameters to get
     */
    public Object[] getMethodParameters() {
        return methodParameters;
    }

    /**
     * @return methodReturn the methodReturn to get
     */
    public Object getMethodReturn() {
        return methodReturn;
    }

    /**
     * @return requestDuration the requestDuration to get
     */
    public double getRequestDuration() {
        return requestDuration;
    }

    /**
     * @return requestStart the requestStart to get
     */
    public long getRequestStart() {
        return requestStart;
    }

    /**
     * @return requestStop the requestStop to get
     */
    public long getRequestStop() {
        return requestStop;
    }

    /**
     * @param requestStartParam the requestStart to set
     */
    public void setRequestStart(final long requestStartParam) {
        this.requestStart = requestStartParam;
    }

    /**
     * @param requestStopParam the requestStop to set
     */
    public void setRequestStop(final long requestStopParam) {
        this.requestStop = requestStopParam;
        this.setRequestDuration((requestStopParam - getRequestStart()) / getNS());
    }

    /**
     * @param requestDurationParam the requestDuration to set
     */
    public void setRequestDuration(final double requestDurationParam) {
        this.requestDuration = requestDurationParam;
    }

    /**
     * @param businessMethodParam the businessMethod to set
     */
    public void setBusinessMethod(final String businessMethodParam) {
        this.businessMethod = businessMethodParam;
    }

    /**
     * @return the target
     */
    public String getTarget() {
        return target;
    }

    /**
     * @param targetParam the target to set
     */
    public void setTarget(final String targetParam) {
        this.target = targetParam;
    }

    /**
     * @param methodReturnParam the methodReturn to set
     */
    public void setMethodReturn(final Object methodReturnParam) {
        this.methodReturn = methodReturnParam;
    }

    /**
     * @param methodParametersParam the method parameters
     */
    public void setMethodParameters(final Object[] methodParametersParam) {
        this.methodParameters = methodParametersParam;
    }

    /**
     * @return the paramSize
     */
    public long getParamSize() {
        return paramSize;
    }

    /**
     * @param paramSize the paramSize to set
     */
    public void setParamSize(final long paramSize) {
        this.paramSize = paramSize;
    }

    /**
     * @return the returnSize
     */
    public long getReturnSize() {
        return returnSize;
    }

    /**
     * @param returnSize the returnSize to set
     */
    public void setReturnSize(final long returnSize) {
        this.returnSize = returnSize;
    }

    /**
     * @return the authType
     */
    public String getAuthType() {
        return authType;
    }

    /**
     * @param authType the authType to set
     */
    public void setAuthType(final String authType) {
        this.authType = authType;
    }

    /**
     * @return the contextPath
     */
    public String getContextPath() {
        return contextPath;
    }

    /**
     * @param contextPath the contextPath to set
     */
    public void setContextPath(final String contextPath) {
        this.contextPath = contextPath;
    }

    /**
     * @return the pathInfo
     */
    public String getPathInfo() {
        return pathInfo;
    }

    /**
     * @param pathInfo the pathInfo to set
     */
    public void setPathInfo(final String pathInfo) {
        this.pathInfo = pathInfo;
    }

    /**
     * @return the pathTranslated
     */
    public String getPathTranslated() {
        return pathTranslated;
    }

    /**
     * @param pathTranslated the pathTranslated to set
     */
    public void setPathTranslated(final String pathTranslated) {
        this.pathTranslated = pathTranslated;
    }

    /**
     * @return the queryString
     */
    public String getQueryString() {
        return queryString;
    }

    /**
     * @param queryString the queryString to set
     */
    public void setQueryString(final String queryString) {
        this.queryString = queryString;
    }

    /**
     * @return the requestedSessionId
     */
    public String getRequestedSessionId() {
        return requestedSessionId;
    }

    /**
     * @param requestedSessionId the requestedSessionId to set
     */
    public void setRequestedSessionId(final String requestedSessionId) {
        this.requestedSessionId = requestedSessionId;
    }

    /**
     * @return the requestURI
     */
    public String getRequestURI() {
        return requestURI;
    }

    /**
     * @param requestURI the requestURI to set
     */
    public void setRequestURI(final String requestURI) {
        this.requestURI = requestURI;
    }

    /**
     * @return the requestURL
     */
    public String getRequestURL() {
        return requestURL;
    }

    /**
     * @param requestURL the requestURL to set
     */
    public void setRequestURL(final String requestURL) {
        this.requestURL = requestURL;
    }

    /**
     * @return the servletPath
     */
    public String getServletPath() {
        return servletPath;
    }

    /**
     * @param servletPath the servletPath to set
     */
    public void setServletPath(final String servletPath) {
        this.servletPath = servletPath;
    }

    /**
     * @return the characterEncoding
     */
    public String getCharacterEncoding() {
        return characterEncoding;
    }

    /**
     * @param characterEncoding the characterEncoding to set
     */
    public void setCharacterEncoding(final String characterEncoding) {
        this.characterEncoding = characterEncoding;
    }

    /**
     * @return the contentLength
     */
    public int getContentLength() {
        return contentLength;
    }

    /**
     * @param contentLength the contentLength to set
     */
    public void setContentLength(final int contentLength) {
        this.contentLength = contentLength;
    }

    /**
     * @return the contentType
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * @param contentType the contentType to set
     */
    public void setContentType(final String contentType) {
        this.contentType = contentType;
    }

    /**
     * @return the localAddress
     */
    public String getLocalAddress() {
        return localAddress;
    }

    /**
     * @param localAddress the localAddress to set
     */
    public void setLocalAddress(final String localAddress) {
        this.localAddress = localAddress;
    }

    /**
     * @return the localName
     */
    public String getLocalName() {
        return localName;
    }

    /**
     * @param localName the localName to set
     */
    public void setLocalName(final String localName) {
        this.localName = localName;
    }

    /**
     * @return the localPort
     */
    public int getLocalPort() {
        return localPort;
    }

    /**
     * @param localPort the localPort to set
     */
    public void setLocalPort(final int localPort) {
        this.localPort = localPort;
    }

    /**
     * @return the protocol
     */
    public String getProtocol() {
        return protocol;
    }

    /**
     * @param protocol the protocol to set
     */
    public void setProtocol(final String protocol) {
        this.protocol = protocol;
    }

    /**
     * @return the remoteAddress
     */
    public String getRemoteAddress() {
        return remoteAddress;
    }

    /**
     * @param remoteAddress the remoteAddress to set
     */
    public void setRemoteAddress(final String remoteAddress) {
        this.remoteAddress = remoteAddress;
    }

    /**
     * @return the remoteHost
     */
    public String getRemoteHost() {
        return remoteHost;
    }

    /**
     * @param remoteHost the remoteHost to set
     */
    public void setRemoteHost(final String remoteHost) {
        this.remoteHost = remoteHost;
    }

    /**
     * @return the remotePort
     */
    public String getRemotePort() {
        return remotePort;
    }

    /**
     * @param remotePort the remotePort to set
     */
    public void setRemotePort(final String remotePort) {
        this.remotePort = remotePort;
    }

    /**
     * @return the scheme
     */
    public String getScheme() {
        return scheme;
    }

    /**
     * @param scheme the scheme to set
     */
    public void setScheme(final String scheme) {
        this.scheme = scheme;
    }

    /**
     * @return the serverName
     */
    public String getServerName() {
        return serverName;
    }

    /**
     * @param serverName the serverName to set
     */
    public void setServerName(final String serverName) {
        this.serverName = serverName;
    }

    /**
     * @return the serverport
     */
    public int getServerport() {
        return serverport;
    }

    /**
     * @param serverport the serverport to set
     */
    public void setServerport(final int serverport) {
        this.serverport = serverport;
    }

    /**
     * @return the secure
     */
    public boolean isSecure() {
        return secure;
    }

    /**
     * @param secure the secure to set
     */
    public void setSecure(final boolean secure) {
        this.secure = secure;
    }

    /**
     * @return the NS
     */
    public static double getNS() {
        return NS;
    }

    /**
     * @return the KB
     */
    public static double getKB() {
        return KB;
    }

    /**
     * @return the HTTP status.
     */
    public int getStatus() {
        return status;
    }

    /**
     * @param status the given status
     */
    public void setStatus(final int status) {
        this.status = status;
    }

}
