/**
 * Copyright 2010-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.auditreport.impl.util;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.ow2.util.auditreport.api.IDisplayableType;
import org.ow2.util.auditreport.api.IEncodableType;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class allow to encode an object into a string and decode it.
 * @author Vincent Michaud
 */
public final class ObjectEncoder {

    /**
     * A map storing all infos of objets encoded classes.
     */
    private static Map<String, FieldInfos> classInfos = new HashMap<String, FieldInfos>();

    /**
     * The prefix for a getter method.
     */
    private static final String GET_PREFIX = "get";

    /**
     * The prefix for a setter method.
     */
    private static final String SET_PREFIX = "set";

    /**
     * The separator for attributes.
     */
    private static final String SEPARATOR_1 = ";,";

    /**
     * The separator for attributes.
     */
    private static final String SEPARATOR_2 = ";#:";

    /**
     * A link to a data at the end of the string.
     */
    private static final String DATA_LINK = SEPARATOR_2;

    /**
     * The delimiter for the value of an attribute.
     */
    private static final String EQUALS = "=";

    /**
     * The null character for an attribute.
     */
    private static final String NULL = ";#[#:";

    /**
     * The text value for a null attribute.
     */
    private static final String NULL_TEXT = "null";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(ObjectEncoder.class);

    /**
     * The private constructor for utility class.
     */
    private ObjectEncoder() {
    }

    /**
     * Load a new object class into the encoder. The class is loaded only if it
     * not already loaded. Fields mapped for encoding are all fields which: <BR />
     * - have a getter and a setter with the name of the field <BR />
     * - are not tables <BR />
     * - are primitive type <BR />
     * - are String <BR />
     * - are Object with a String constructor <BR />
     * - are registred with a EZBEncodableType action
     * @param objectClass The object class
     * @return The list of mapped field in the class, or null if an error occurs
     */
    private static FieldInfos loadClass(final Class<?> objectClass) {
        FieldInfos fieldInfos = classInfos.get(objectClass.getName());
        if (fieldInfos == null) {
            fieldInfos = new FieldInfos();
            Method getMethod, setMethod;
            String fieldName = null;
            FieldInfo fieldInfo;
            Class<?> fieldType;

            if (!haveDefaultConstructor(objectClass)) {
                logger.error("Error while loading class : " + objectClass.getName());
                logger.error("    No default constructor found.");
                return null;
            }

            for (Field field : objectClass.getDeclaredFields()) {
                try {
                    // If a set and a get methods are identified on the field
                    fieldName = field.getName();
                    fieldType = field.getType();
                    getMethod = objectClass.getMethod(getFieldMethodName(fieldName, GET_PREFIX));
                    setMethod = objectClass.getMethod(getFieldMethodName(fieldName, SET_PREFIX), fieldType);

                    if (!fieldType.isArray()) {
                        if (fieldType.isPrimitive()) {
                            fieldType = checkType(fieldType);
                            fieldInfo = new FieldInfo(fieldName, fieldType, null, getMethod, setMethod);
                            fieldInfos.put(fieldName, fieldInfo);
                        } else if (haveStringConstructor(fieldType)) {
                            fieldInfo = new FieldInfo(fieldName, fieldType, null, getMethod, setMethod);
                            fieldInfos.put(fieldName, fieldInfo);
                        }
                    }

                } catch (NoSuchMethodException ex) {
                    logger.debug("Field " + fieldName + " will not be parsed.");
                } catch (Exception ex) {
                    logger.error("Error while analyzing the class: " + objectClass.getName() + ".");
                    ex.printStackTrace();
                }
            }

            Class<?> currentClass = objectClass.getSuperclass();
            FieldInfos currentFieldInfos;

            while (currentClass != null) {
                currentFieldInfos = loadClass(currentClass);
                for (FieldInfo info : currentFieldInfos.order) {
                    fieldInfos.put(info.fieldName, info);
                }
                currentClass = currentClass.getSuperclass();
            }

            classInfos.put(objectClass.getName(), fieldInfos);
        }
        return fieldInfos;
    }

    /**
     * Encode an object into a string.
     * @param object The object to encode
     * @return The string, or null if an error occurs
     */
    public static String encode(final Object object) {
        Class<?> objectClass = object.getClass();
        FieldInfos fieldInfos = loadClass(objectClass);
        if (fieldInfos != null) {
            List<FieldInfo> listInfos = fieldInfos.values();
            StringBuilder build = new StringBuilder();
            StringBuilder endStr = new StringBuilder();
            String fieldString, encodedObject;
            Object fieldData;

            for (FieldInfo fieldInfo : listInfos) {
                try {
                    fieldData = fieldInfo.getMethod.invoke(object);

                    if (fieldData == null) {
                        fieldString = NULL;
                    } else {
                        if (fieldInfo.conversion == null) {
                            if (fieldInfo.fieldType == String.class) {
                                fieldString = (String) fieldData;
                            } else {
                                fieldString = fieldData.toString();
                            }
                        } else {
                            fieldString = DATA_LINK;
                            endStr.append(fieldInfo.conversion.toString(fieldData));
                            endStr.append(SEPARATOR_2);
                        }
                    }
                    build.append(fieldInfo.fieldName);
                    build.append(EQUALS);
                    build.append(fieldString);
                    build.append(SEPARATOR_1);
                } catch (Exception ex) {
                    logger.error("Error while encoding the object" + object.getClass().getName() + " @ " + fieldInfo.fieldName
                            + " : " + ex);
                    ex.printStackTrace();
                }
            }
            if (listInfos.size() != 0) {
                build.setLength(build.length() - SEPARATOR_1.length());
            }
            if (endStr.length() != 0) {
                endStr.setLength(endStr.length() - SEPARATOR_2.length());
            }

            encodedObject = build.toString();
            build.setLength(0);
            build.append(encodedObject.length());
            build.append(SEPARATOR_1);
            build.append(objectClass.getName());
            build.append(SEPARATOR_1);
            build.append(encodedObject);
            build.append(endStr);
            return build.toString();
        }
        return null;
    }

    /**
     * Decode a string and convert it to the initial object.
     * @param encodedObject The encoded object in a String format
     * @return The decoded object, or null if an error occurs
     */
    public static Object decode(final String encodedObject) {
        EncodedObjectReader reader = null;
        String[] objValues, property = null;
        FieldInfos fieldInfos;
        Object decodedObject;

        // Analyze the message: extract the class and field values of the
        // object.
        try {
            reader = new EncodedObjectReader(encodedObject);
            objValues = reader.fieldSpecialValues;
            Class classObject = Class.forName(reader.className);
            decodedObject = classObject.newInstance();
            fieldInfos = loadClass(classObject);
        } catch (Exception ex) {
            logger.error("Error while decoding the object : \n");
            logger.error("    " + ex.getClass().getName() + " : " + ex.getMessage());
            return null;
        }

        // Create the object and set the value of fields.
        if (fieldInfos != null) {
            Object fieldValue;
            FieldInfo fieldInfo;
            int nbRegistredObj = 0;

            for (String param : reader.fieldProperties) {
                try {
                    property = createProperty(param);
                    fieldInfo = fieldInfos.get(property[0]);
                    if (fieldInfo != null) {
                        if (NULL.equals(property[1])) {
                            fieldInfo.setMethod.invoke(decodedObject, (Object) null);
                        } else {
                            if (fieldInfo.conversion == null) {
                                if (fieldInfo.fieldType == String.class) {
                                    fieldInfo.setMethod.invoke(decodedObject, property[1]);
                                } else {
                                    Constructor c = fieldInfo.fieldType.getConstructor(String.class);
                                    fieldInfo.setMethod.invoke(decodedObject, c.newInstance(property[1]));
                                }
                            } else if (DATA_LINK.equals(property[1]) && nbRegistredObj < objValues.length) {
                                fieldValue = fieldInfo.conversion.fromString(objValues[nbRegistredObj]);
                                nbRegistredObj++;
                                fieldInfo.setMethod.invoke(decodedObject, fieldValue);
                            }
                        }
                    }
                } catch (Exception ex) {
                    logger.error("Error : Can not set the value of the field " + property[0] + ".");
                }
            }
            return decodedObject;
        }
        return null;
    }

    /**
     * Decode a string and convert it to the specified object type. All missing
     * fields are added into lists of EZBEncodableClass object.
     * @param <T> The type of the decoded object created
     * @param objectClass The object type
     * @param encodedObject The encoded object in a String format
     * @return The decoded object, or null if an error occurs
     */
    public static <T> T decode(final String encodedObject, final Class<T> objectClass) {
        return decode(encodedObject, objectClass, null, null);
    }

    /**
     * Decode a string and convert it to the specified object type. All missing
     * fields are added into lists of EZBEncodableClass object.
     * @param <T> The type of the decoded object created
     * @param objectClass The object type (Can be null)
     * @param encodedObject The encoded object in a String format
     * @param fieldNames A list where are stored names of all unknown fields. It
     *        is recommended that list will be empty at the call of the
     *        function. (Can be null)
     * @param fieldValues A list where are stored values of all unknown fields.
     *        It is recommended that list will be empty at the call of the
     *        function. (Can be null)
     * @return The decoded object, or null if an error occurs
     */
    public static <T> T decode(final String encodedObject, final Class<T> objectClass, final List<String> fieldNames,
            final List<String> fieldValues) {
        EncodedObjectReader reader = null;
        String[] objValues, property = null;
        FieldInfos fieldInfos = null;
        T decodedObject = null;

        // Analyze the message: extract the class and field values of the
        // object.
        try {
            reader = new EncodedObjectReader(encodedObject);
            objValues = reader.fieldSpecialValues;
            if (objectClass != null) {
                decodedObject = objectClass.newInstance();
                fieldInfos = loadClass(objectClass);
            }
        } catch (Exception ex) {
            logger.error("Error while decoding the object : \n");
            logger.error("    " + ex.getClass().getName() + " : " + ex.getMessage());
            return null;
        }

        // Create the object and set the value of fields.
        Object fieldValue;
        FieldInfo fieldInfo = null;
        int nbRegistredObj = 0;
        boolean storeInList = (fieldNames != null && fieldValues != null);

        for (String param : reader.fieldProperties) {
            try {
                property = createProperty(param);
                if (fieldInfos != null) {
                    fieldInfo = fieldInfos.get(property[0]);
                }

                if (fieldInfo != null) {
                    if (NULL.equals(property[1])) {
                        fieldInfo.setMethod.invoke(decodedObject, (Object) null);
                    } else {
                        if (fieldInfo.conversion == null) {
                            if (fieldInfo.fieldType == String.class) {
                                fieldInfo.setMethod.invoke(decodedObject, property[1]);
                            } else {
                                Constructor c = fieldInfo.fieldType.getConstructor(String.class);
                                fieldInfo.setMethod.invoke(decodedObject, c.newInstance(property[1]));
                            }
                        } else if (DATA_LINK.equals(property[1]) && nbRegistredObj < objValues.length) {
                            fieldValue = fieldInfo.conversion.fromString(objValues[nbRegistredObj]);
                            nbRegistredObj++;
                            fieldInfo.setMethod.invoke(decodedObject, fieldValue);
                        }
                    }
                } else if (storeInList) {
                    fieldNames.add(property[0]);
                    if (NULL.equals(property[1])) {
                        fieldValues.add(NULL_TEXT);
                    } else if (DATA_LINK.equals(property[1]) && nbRegistredObj < objValues.length) {
                        fieldValues.add(objValues[nbRegistredObj]);
                        nbRegistredObj++;
                    } else {
                        fieldValues.add(property[1]);
                    }
                }
            } catch (Exception ex) {
                logger.error("Error : Can not set the value of the field " + property[0] + ".");
            }
        }
        return decodedObject;
    }

    /**
     * Decode a string and convert it to the specified object type. All missing
     * fields are added into lists of EZBEncodableClass object.
     * @param <T> The type of the decoded object created
     * @param objectClass The object type (Can be null)
     * @param encodedObject The encoded object in a String format
     * @param fieldNames A list where are stored names of all unknown fields. It
     *        is recommended that list will be empty at the call of the
     *        function. (Can be null)
     * @param fieldValues A list where are stored values of all unknown fields.
     *        It is recommended that list will be empty at the call of the
     *        function. (Can be null)
     * @return The decoded object, or null if an error occurs
     */
    public static <T> T decodeToDisplayable(final String encodedObject, final Class<T> objectClass,
            final List<String> fieldNames, final List fieldValues) {
        EncodedObjectReader reader = null;
        String[] objValues, property = null;
        FieldInfos realFieldInfos, fieldInfos = null;
        T decodedObject = null;

        // Analyze the message: extract the class and field values of the
        // object.
        try {
            reader = new EncodedObjectReader(encodedObject);
            objValues = reader.fieldSpecialValues;
            Class classObject = null;
            try {
                classObject = Class.forName(reader.className);
            } catch (ClassNotFoundException e) {
                classObject = objectClass.getClassLoader().loadClass(reader.className);
            }
            realFieldInfos = loadClass(classObject);
            if (objectClass != null) {
                decodedObject = objectClass.newInstance();
                fieldInfos = loadClass(objectClass);
            }
        } catch (Exception ex) {
            logger.error("Error while decoding the object : \n");
            logger.error("    " + ex.getClass().getName() + " : " + ex.getMessage());
            return null;
        }

        // Create the object and set the value of fields.
        Object fieldValue, displayable;
        FieldInfo realFieldInfo, fieldInfo = null;
        int nbRegistredObj = 0;

        for (String param : reader.fieldProperties) {
            try {
                property = createProperty(param);
                realFieldInfo = realFieldInfos.get(property[0]);
                fieldNames.add(property[0]);
                if (fieldInfos != null) {
                    fieldInfo = fieldInfos.get(property[0]);
                }

                if (NULL.equals(property[1])) {
                    fieldValues.add(NULL_TEXT);
                    if (fieldInfo != null) {
                        fieldInfo.setMethod.invoke(decodedObject, (Object) null);
                    }
                } else if (fieldInfo != null) {
                    if (fieldInfo.conversion == null) {
                        fieldValues.add(property[1]);
                        if (fieldInfo.fieldType == String.class) {
                            fieldInfo.setMethod.invoke(decodedObject, property[1]);
                        } else {
                            Constructor c = fieldInfo.fieldType.getConstructor(String.class);
                            fieldInfo.setMethod.invoke(decodedObject, c.newInstance(property[1]));
                        }
                    } else if (DATA_LINK.equals(property[1]) && nbRegistredObj < objValues.length) {
                        if (realFieldInfo.conversion instanceof IDisplayableType) {
                            IDisplayableType dt = (IDisplayableType) realFieldInfo.conversion;
                            displayable = dt.encodedToDisplayable(objValues[nbRegistredObj]);
                            fieldValues.add(displayable);
                        } else {
                            fieldValues.add(objValues[nbRegistredObj]);
                        }
                        fieldValue = fieldInfo.conversion.fromString(objValues[nbRegistredObj]);
                        nbRegistredObj++;
                        fieldInfo.setMethod.invoke(decodedObject, fieldValue);
                    }
                } else if (DATA_LINK.equals(property[1]) && nbRegistredObj < objValues.length) {
                    if (realFieldInfo.conversion instanceof IDisplayableType) {
                        IDisplayableType dt = (IDisplayableType) realFieldInfo.conversion;
                        displayable = dt.encodedToDisplayable(objValues[nbRegistredObj]);
                        fieldValues.add(displayable);
                    } else {
                        fieldValues.add(objValues[nbRegistredObj]);
                    }
                    nbRegistredObj++;
                } else {
                    fieldValues.add(property[1]);
                }
            } catch (Exception ex) {
                logger.error("Error : Can not set the value of the field " + property[0] + ".");
            }
        }
        return decodedObject;
    }

    /**
     * Encode an object which is on its displayable representation.
     * @param objectClass The class of the object to encode
     * @param fieldNames A list where are stored names of fields
     * @param fieldValues A list where are stored values of fields. Values can
     *        be a string or a list of string (in case of array, or complex
     *        objects)
     * @return The encoded object
     */
    public static String encodeFromDisplayable(final Class<?> objectClass, final List<String> fieldNames, final List fieldValues) {
        if (fieldNames.size() != fieldValues.size()) {
            return null;
        }

        FieldInfos fieldInfos = loadClass(objectClass);
        if (fieldInfos != null) {
            StringBuilder build = new StringBuilder();
            StringBuilder endStr = new StringBuilder();
            String fieldString, encodedObject;

            Object fieldData;
            FieldInfo fieldInfo;
            Iterator it = fieldValues.iterator();

            for (String fieldName : fieldNames) {
                fieldInfo = fieldInfos.get(fieldName);
                fieldData = it.next();

                if (fieldInfo != null) {
                    if (fieldData.equals(NULL_TEXT)) {
                        fieldString = NULL;
                    } else if (fieldInfo.conversion != null) {
                        fieldString = DATA_LINK;
                        if (fieldInfo.conversion instanceof IDisplayableType) {
                            endStr.append(((IDisplayableType) fieldInfo.conversion).displayableToEncoded(fieldData));
                        } else {
                            endStr.append((String) fieldData);
                        }
                        endStr.append(SEPARATOR_2);
                    } else if (fieldData instanceof String) {
                        fieldString = (String) fieldData;
                    } else {
                        fieldString = fieldData.toString();
                    }

                    build.append(fieldName);
                    build.append(EQUALS);
                    build.append(fieldString);
                    build.append(SEPARATOR_1);
                }
            }

            if (fieldNames.size() != 0) {
                build.setLength(build.length() - SEPARATOR_1.length());
            }
            if (endStr.length() != 0) {
                endStr.setLength(endStr.length() - SEPARATOR_2.length());
            }

            encodedObject = build.toString();
            build.setLength(0);
            build.append(encodedObject.length());
            build.append(SEPARATOR_1);
            build.append(objectClass.getName());
            build.append(SEPARATOR_1);
            build.append(encodedObject);
            build.append(endStr);
            return build.toString();
        }
        return null;
    }

    /**
     * Register a field of a class which is not mapped by default by the
     * encoder. After the registration, the field will be present when the
     * encoding / decoding of the object thanks to the EZBEncodableType action
     * specified. If the field is already registred, the EZBEncodableType action
     * is replaced. A call to this function MUST be done in a static bloc of the
     * class.
     * @param typeClass The class
     * @param fieldName The name of the field
     * @param action The action done when the encoding / decoding process
     */
    public static void registerField(final Class typeClass, final String fieldName, final IEncodableType action) {
        FieldInfos fieldInfos = loadClass(typeClass);
        if (action != null && fieldInfos != null) {
            FieldInfo fieldInfo = fieldInfos.get(fieldName);

            if (fieldInfo == null) {
                String getterName = getFieldMethodName(fieldName, GET_PREFIX);
                String setterName = getFieldMethodName(fieldName, SET_PREFIX);
                try {
                    Class fieldType = typeClass.getDeclaredField(fieldName).getType();
                    Method getMethod = typeClass.getMethod(getterName);
                    Method setMethod = typeClass.getMethod(setterName, fieldType);
                    fieldInfo = new FieldInfo(fieldName, fieldType, action, getMethod, setMethod);
                    fieldInfos.put(fieldName, fieldInfo);
                } catch (NoSuchFieldException ex) {
                    logger.error("Error while registring the field " + fieldName + ". This field does not exist.");
                } catch (NoSuchMethodException ex) {
                    logger.error("Error while registring the field " + fieldName
                            + ". This field must have a getter and a setter: " + getterName + "() and " + setterName + "().");
                }
            } else if (action != null) {
                fieldInfo.conversion = action;
            }
        }
    }

    /**
     * Unregister a field of a class which is mapped by the encoder. A call to
     * this function MUST be done in a static bloc of the class.
     * @param typeClass The class
     * @param fieldName The name of the field
     */
    public static void unregisterField(final Class typeClass, final String fieldName) {
        FieldInfos fieldInfos = loadClass(typeClass);
        if (fieldInfos != null) {
            fieldInfos.remove(fieldName);
        }
    }

    /**
     * Get the clas name of the encoded object.
     * @param encodedObject The encoded object
     * @return The class name
     */
    public static String getClassName(final String encodedObject) {
        int index = encodedObject.indexOf(SEPARATOR_1) + SEPARATOR_1.length();
        int index2 = encodedObject.indexOf(SEPARATOR_1, index);
        return encodedObject.substring(index, index2);
    }

    /**
     * Get the conversion methods of a specific field in the specified class.
     * @param className The name of the class
     * @param fieldName The name of the field
     * @return The EZBEncodableType found, or null if nothing is found
     */
    public static IEncodableType getEncodableType(final String className, final String fieldName) {
        FieldInfos fieldInfos = classInfos.get(className);
        if (fieldInfos != null) {
            FieldInfo fieldInfo = fieldInfos.get(fieldName);
            if (fieldInfo != null) {
                return fieldInfo.conversion;
            }
        }
        return null;
    }

    /**
     * Get the name of a getter or a setter of a field.
     * @param fieldName The name of the field
     * @param prefix The prefix of the method (get / set)
     * @return The name of the method
     */
    private static String getFieldMethodName(final String fieldName, final String prefix) {
        StringBuilder builder = new StringBuilder(prefix);
        builder.append(String.valueOf(fieldName.charAt(0)).toUpperCase());
        builder.append(fieldName.substring(1));
        return builder.toString();
    }

    /**
     * Determine if a class have a String constructor.
     * @param typeClass The class
     * @return True if a String constructor exist, false otherwise
     */
    private static boolean haveStringConstructor(final Class typeClass) {
        try {
            typeClass.getConstructor(String.class);
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    /**
     * Determine if a class have a default constructor.
     * @param typeClass The class
     * @return True if a default constructor exist, false otherwise
     */
    private static boolean haveDefaultConstructor(final Class typeClass) {
        try {
            typeClass.getConstructor();
        } catch (Exception ex) {
            return false;
        }
        return true;
    }

    /**
     * Create an array from a parameter.
     * @param param The parameter
     * @return The array
     */
    private static String[] createProperty(final String param) {
        int index = param.indexOf(EQUALS);
        String[] tab = {param.substring(0, index), param.substring(index + EQUALS.length())};
        return tab;
    }

    /**
     * This method transform a primitive type class into its object class.
     * @param typeClass the primitive class.
     * @return the object class.
     */
    private static Class checkType(final Class typeClass) {
        if (typeClass == int.class) {
            return Integer.class;
        } else if (typeClass == boolean.class) {
            return Boolean.class;
        } else if (typeClass == long.class) {
            return Long.class;
        } else if (typeClass == double.class) {
            return Double.class;
        } else if (typeClass == byte.class) {
            return Byte.class;
        } else if (typeClass == float.class) {
            return Float.class;
        } else if (typeClass == short.class) {
            return Short.class;
        }
        return typeClass;
    }

    /**
     * Read an encoded object.
     */
    static class EncodedObjectReader {

        /**
         * The name of the class.
         */
        private String className;

        /**
         * The list of field properties. Ex: "timestamp=1591121545"
         */
        private String[] fieldProperties;

        /**
         * The list of field values which have registred EZBEncodableClass.
         */
        private String[] fieldSpecialValues;

        /**
         * Constructor.
         * @param encodedObject The encoded object to read
         */
        public EncodedObjectReader(final String encodedObject) {
            int index = encodedObject.indexOf(SEPARATOR_1);
            int lengthMessage = Integer.parseInt(encodedObject.substring(0, index));
            index += SEPARATOR_1.length();
            int index2 = encodedObject.indexOf(SEPARATOR_1, index);
            this.className = encodedObject.substring(index, index2);
            index = index2 + SEPARATOR_1.length();
            index2 = index + lengthMessage;
            this.fieldProperties = encodedObject.substring(index, index2).split(SEPARATOR_1);
            this.fieldSpecialValues = encodedObject.substring(index2).split(SEPARATOR_2);
        }

        /**
         * Get the name of the class of the encoded object.
         * @return The name of the class
         */
        public String getClassName() {
            return this.className;
        }

        /**
         * Get the list of field properties. A field property is composed of the
         * name of the field and its value. Ex: "timestamp=1591121545"
         * @return The field properties
         */
        public String[] getFieldProperties() {
            return this.fieldProperties;
        }

        /**
         * Get the list of values of field which have been registred with a
         * EZBEncodableClass.
         * @return The special field values
         */
        public String[] getFieldSpecialValues() {
            return this.fieldSpecialValues;
        }
    }

    /**
     * A simple representation of a field with its encoding and decoding action.
     */
    static class FieldInfo {

        /**
         * The name of the field.
         */
        private String fieldName;

        /**
         * The type of the field.
         */
        private Class fieldType;

        /**
         * The action used to encode and decode a field.
         */
        private IEncodableType conversion;

        /**
         * The method called for getting the value of the field.
         */
        private Method getMethod;

        /**
         * The method called for setting the value of the field.
         */
        private Method setMethod;

        /**
         * Constructor.
         * @param fieldName The name of the field
         * @param fieldType The type of the field
         * @param conversion The action used to encode and decode a field
         * @param getMethod The method called for getting the value of the field
         * @param setMethod The method called for setting the value of the field
         */
        public FieldInfo(final String fieldName, final Class fieldType, final IEncodableType conversion,
                final Method getMethod, final Method setMethod) {
            this.fieldName = fieldName;
            this.fieldType = fieldType;
            this.conversion = conversion;
            this.getMethod = getMethod;
            this.setMethod = setMethod;
        }

        /**
         * Get a representative string of the object.
         * @return A string
         */
        @Override
        public String toString() {
            return this.fieldName + " : " + this.fieldType;
        }
    }

    /**
     * Contains all FieldInfo data.
     */
    static class FieldInfos {

        /**
         * The order beetween fields.
         */
        private List<FieldInfo> order;

        /**
         * A quick access to fields.
         */
        private HashMap<String, FieldInfo> map;

        /**
         * Constructor.
         */
        public FieldInfos() {
            this.map = new HashMap<String, FieldInfo>();
            this.order = new ArrayList<FieldInfo>();
        }

        /**
         * Add a new FieldInfo in the structure.
         * @param fieldName The name of the field
         * @param fieldInfo The field information
         */
        public void put(final String fieldName, final FieldInfo fieldInfo) {
            if (!this.map.containsKey(fieldName)) {
                this.order.add(fieldInfo);
                this.map.put(fieldName, fieldInfo);
            }
        }

        /**
         * Get a field information.
         * @param fieldName The name of the field
         * @return The field information
         */
        public FieldInfo get(final String fieldName) {
            return this.map.get(fieldName);
        }

        /**
         * Remove a field information.
         * @param fieldName The name of the field
         * @return The removed field information, or null, if nothing is removed
         */
        public FieldInfo remove(final String fieldName) {
            FieldInfo removed = this.map.remove(fieldName);
            if (removed != null) {
                Iterator<FieldInfo> it = this.order.iterator();
                FieldInfo fieldInfo = null;
                while (it.hasNext()) {
                    fieldInfo = it.next();
                    if (fieldInfo.fieldName.equals(fieldName)) {
                        it.remove();
                        return removed;
                    }
                }
            }
            return null;
        }

        /**
         * List all field information, in the declaration order.
         * @return The list
         */
        public List<FieldInfo> values() {
            return this.order;
        }
    }
}
